==> SimpleMailer-1.09.tar.bz2: SimpleMailer-1.09/core/Smdn.Formats.Mime/Smdn.Formats.Mime/Mail.cs <==
//
// Author:
//       smdn <smdn@smdn.jp>
//
// Copyright (c) 2008-2014 smdn
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
#if NET_4_0
// LazyInitializer
using System.Threading;
#else
using Smdn.Threading;
#endif
using System.IO;
using System.Text;

namespace Smdn.Formats.Mime {
  public class Mail : MailBase, IMimeMessage, IDisposable {
    /*
     * class members
     */


    /*
     * Load(file)
     */
    public static Mail Load(string file)
    {
      return new Mail(MimeMessage.Load(file));
    }

    public static Mail Load(string file,
                            bool loadContentToMemory)
    {
      return new Mail(MimeMessage.Load(file,
                                       loadContentToMemory));
    }

    public static Mail Load(string file,
                            Encoding fallbackCharset)
    {
      return new Mail(MimeMessage.Load(file,
                                       fallbackCharset));
    }

    public static Mail Load(string file,
                            bool loadContentToMemory,
                            Encoding fallbackCharset)
    {
      return new Mail(MimeMessage.Load(file,
                                       loadContentToMemory,
                                       fallbackCharset));
    }

    public static Mail Load(string file,
                            EncodingSelectionCallback selectFallbackCharset)
    {
      return new Mail(MimeMessage.Load(file,
                                       selectFallbackCharset));
    }

    public static Mail Load(string file,
                            bool loadContentToMemory,
                            EncodingSelectionCallback selectFallbackCharset)
    {
      return new Mail(MimeMessage.Load(file,
                                       loadContentToMemory,
                                       selectFallbackCharset));
    }

    public static Mail Load(string file,
                            EncodingSelectionCallback selectFallbackCharset,
                            MimeEncodedWordConverter decodeMalformedHeaderValue)
    {
      return new Mail(MimeMessage.Load(file,
                                       selectFallbackCharset,
                                       decodeMalformedHeaderValue));
    }

    public static Mail Load(string file,
                            bool loadContentToMemory,
                            EncodingSelectionCallback selectFallbackCharset,
                            MimeEncodedWordConverter decodeMalformedHeaderValue)
    {
      return new Mail(MimeMessage.Load(file,
                                       loadContentToMemory,
                                       selectFallbackCharset,
                                       decodeMalformedHeaderValue));
    }

    /*
     * Load(stream)
     */
    public static Mail Load(Stream stream)
    {
      return new Mail(MimeMessage.Load(stream));
    }

    public static Mail Load(Stream stream,
                            Encoding fallbackCharset)
    {
      return new Mail(MimeMessage.Load(stream,
                                       fallbackCharset));
    }

    public static Mail Load(Stream stream,
                            EncodingSelectionCallback selectFallbackCharset)
    {
      return new Mail(MimeMessage.Load(stream,
                                       selectFallbackCharset));
    }

    public static Mail Load(Stream stream,
                            EncodingSelectionCallback selectFallbackCharset,
                            MimeEncodedWordConverter decodeMalformedHeaderValue)
    {
      return new Mail(MimeMessage.Load(stream,
                                       selectFallbackCharset,
                                       decodeMalformedHeaderValue));
    }

    /*
     * LoadMail
     */
    public static Mail LoadMail(string mail)
    {
      return new Mail(MimeMessage.LoadMessage(mail));
    }

    public static Mail LoadMail(string mail,
                                Encoding fallbackCharset)
    {
      return new Mail(MimeMessage.LoadMessage(mail,
                                              fallbackCharset));
    }

    public static Mail LoadMail(string mail,
                                EncodingSelectionCallback selectFallbackCharset)
    {
      return new Mail(MimeMessage.LoadMessage(mail,
                                              selectFallbackCharset));
    }

    public static Mail LoadMail(string mail,
                                EncodingSelectionCallback selectFallbackCharset,
                                MimeEncodedWordConverter decodeMalformedHeaderValue)
    {
      return new Mail(MimeMessage.LoadMessage(mail,
                                              selectFallbackCharset,
                                              decodeMalformedHeaderValue));
    }

    /*
     * instance members
     */
    public MimeHeaderCollection Headers {
      get { return BaseMessage.Headers; }
    }

    public MimeType MimeType {
      get { return BaseMessage.MimeType; }
    }

    public MimeMessageDisposition Disposition {
      get { return BaseMessage.Disposition; }
    }

    public string Subject {
      get { return GetHeaderValueOf("Subject"); }
    }

    public string From {
      get { return GetHeaderValueOf("From"); }
    }

    public string To {
      get { return GetHeaderValueOf("To"); }
    }

    public string DateString {
      get { return GetHeaderValueOf("Date"); }
    }

    public DateTimeOffset? Date {
      get {
        try {
          return DateTimeConvert.FromRFC822DateTimeOffsetStringNullable(GetHeaderValueOf("Date"));
        }
        catch (FormatException) {
          return null;
        }
      }
    }

    public string MessageId {
      get { return GetHeaderValueOf("Message-ID"); }
    }

    public string InReplyTo {
      get { return GetHeaderValueOf("In-Reply-To"); }
    }

    public string References {
      get { return GetHeaderValueOf("References"); }
    }

    /// <returns>header value of X-Mailer or User-Agent</returns>
    public string UserAgent {
      get { return GetHeaderValueOf("X-Mailer") ?? GetHeaderValueOf("User-Agent"); }
    }

    public string ReplyTo {
      get { return GetHeaderValueOf("Reply-To"); }
    }

    public string Sender {
      get { return GetHeaderValueOf("Sender"); }
    }

#if false
    public string[] CC {
      get { return GetHeaderValueOf("CC").Split(','); }
    }

    public string[] Bcc {
      get { return GetHeaderValueOf("BCC").Split(','); }
    }
#endif

    public ReadOnlyCollection<MimeMessage> SubParts {
      get { return BaseMessage.SubParts; }
    }

    MimeMessage IMimeMessage.ParentPart {
      get { return BaseMessage.ParentPart; }
    }

    MimeMessage IMimeMessage.RootPart {
      get { return BaseMessage.RootPart; }
    }

    public MimeMessage EncapsulatedMessage {
      get { return BaseMessage.EncapsulatedMessage; }
    }

    public IEnumerable<MimeMessage> AttachmentParts {
      get {
        return BaseMessage.SubParts
          .Where(p => p.Disposition == MimeMessageDisposition.Attachment);
      }
    }

    public IEnumerable<MimeMessage> InlineMessageParts {
      get {
        return BaseMessage.SubParts
          .Where(p => p.Disposition == MimeMessageDisposition.Inline);
      }
    }

    private ReadOnlyCollection<AttachedFile> attachedFiles;

    public ReadOnlyCollection<AttachedFile> AttachedFiles {
      get {
        return LazyInitializer.EnsureInitialized(ref attachedFiles, delegate {
          return BaseMessage
            .GetFlattenSubParts()
            .Where(IsAttachedFile)
            .Select(p => new AttachedFile(this, p))
            .ToList()
            .AsReadOnly();
        });
      }
    }

    public Mail(MimeMessage baseMessage)
      : base(baseMessage)
    {
    }

    private static bool IsAttachedFile(MimeMessage part)
    {
      if (part.MimeType.TypeEqualsIgnoreCase("multipart"))
        return false;
      if (MimeType.MessagePartial.EqualsIgnoreCase(part.MimeType))
        return false;

      if (part.ParentPart != null) {
        if (part.Disposition == MimeMessageDisposition.Attachment)
          return true;

        // nested part
        if (part.ParentPart == part.RootPart &&
            part.ParentPart.MimeType.EqualsIgnoreCase("multipart/mixed") &&
            part.MimeType.EqualsIgnoreCase("text/plain") || part.MimeType.EqualsIgnoreCase("text/html"))
          // text part under root part
          return false;

        if (part.ParentPart.MimeType.EqualsIgnoreCase("multipart/alternative"))
          // alternative part
          return false;
      }

      // other
      return true;
    }

    public void Dispose()
    {
      DisposeCore();
    }

    public string GetHeaderValueOf(string headerName)
    {
      return GetHeaderValueOf(headerName, true);
    }

    public string GetHeaderValueOf(string headerName, bool withParameter)
    {
      var header = BaseMessage.Headers.GetHeaders(headerName).FirstOrDefault();

      if (header == null) {
        return null;
      }
      else {
        if (withParameter)
          return header.Value;
        else
          return header.GetValueWithoutParameter();
      }
    }

    public FileInfo[] SaveAttachedFilesToDirectory(string directory)
    {
      if (directory == null)
        throw new ArgumentNullException("directory");
      if (directory.Length == 0)
        throw ExceptionUtils.CreateArgumentMustBeNonEmptyString("directory");

      CheckDisposed();

      if (!Directory.Exists(directory))
        Directory.CreateDirectory(directory);

      return SaveAttachedFilesAs(delegate(AttachedFile attachedFile) {
        return Path.Combine(directory, attachedFile.GetDefaultFileName());
      });
    }

    public FileInfo[] SaveAttachedFilesAs(Func<AttachedFile, string> selectFileName)
    {
      if (selectFileName == null)
        throw new ArgumentNullException("selectFileName");

      return AttachedFiles.Select(delegate(AttachedFile attachedFile) {
        var file = new FileInfo(selectFileName(attachedFile));

        attachedFile.SaveAs(file);

        return file;
      }).ToArray();
    }
  }
}

==> SimpleMailer-1.11.tar.bz2: SimpleMailer-1.11/core/Smdn.Formats.Mime/Smdn.Formats.Mime/Mail.cs <==
//
// Author:
//       smdn <smdn@smdn.jp>
//
// Copyright (c) 2008-2014 smdn
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
#if NET_4_0
// LazyInitializer
using System.Threading;
#else
using Smdn.Threading;
#endif
using System.IO;
using System.Text;

namespace Smdn.Formats.Mime {
  public class Mail : MailBase, IMimeMessage, IDisposable {
    /*
     * class members
     */


    /*
     * Load(file)
     */
    public static Mail Load(string file)
    {
      return new Mail(MimeMessage.Load(file));
    }

    public static Mail Load(string file,
                            bool loadContentToMemory)
    {
      return new Mail(MimeMessage.Load(file,
                                       loadContentToMemory));
    }

    public static Mail Load(string file,
                            Encoding fallbackCharset)
    {
      return new Mail(MimeMessage.Load(file,
                                       fallbackCharset));
    }

    public static Mail Load(string file,
                            bool loadContentToMemory,
                            Encoding fallbackCharset)
    {
      return new Mail(MimeMessage.Load(file,
                                       loadContentToMemory,
                                       fallbackCharset));
    }

    public static Mail Load(string file,
                            EncodingSelectionCallback selectFallbackCharset)
    {
      return new Mail(MimeMessage.Load(file,
                                       selectFallbackCharset));
    }

    public static Mail Load(string file,
                            bool loadContentToMemory,
                            EncodingSelectionCallback selectFallbackCharset)
    {
      return new Mail(MimeMessage.Load(file,
                                       loadContentToMemory,
                                       selectFallbackCharset));
    }

    public static Mail Load(string file,
                            EncodingSelectionCallback selectFallbackCharset,
                            MimeEncodedWordConverter decodeMalformedHeaderValue)
    {
      return new Mail(MimeMessage.Load(file,
                                       selectFallbackCharset,
                                       decodeMalformedHeaderValue));
    }

    public static Mail Load(string file,
                            bool loadContentToMemory,
                            EncodingSelectionCallback selectFallbackCharset,
                            MimeEncodedWordConverter decodeMalformedHeaderValue)
    {
      return new Mail(MimeMessage.Load(file,
                                       loadContentToMemory,
                                       selectFallbackCharset,
                                       decodeMalformedHeaderValue));
    }

    /*
     * Load(stream)
     */
    public static Mail Load(Stream stream)
    {
      return new Mail(MimeMessage.Load(stream));
    }

    public static Mail Load(Stream stream,
                            Encoding fallbackCharset)
    {
      return new Mail(MimeMessage.Load(stream,
                                       fallbackCharset));
    }

    public static Mail Load(Stream stream,
                            EncodingSelectionCallback selectFallbackCharset)
    {
      return new Mail(MimeMessage.Load(stream,
                                       selectFallbackCharset));
    }

    public static Mail Load(Stream stream,
                            EncodingSelectionCallback selectFallbackCharset,
                            MimeEncodedWordConverter decodeMalformedHeaderValue)
    {
      return new Mail(MimeMessage.Load(stream,
                                       selectFallbackCharset,
                                       decodeMalformedHeaderValue));
    }

    /*
     * LoadMail
     */
    public static Mail LoadMail(string mail)
    {
      return new Mail(MimeMessage.LoadMessage(mail));
    }

    public static Mail LoadMail(string mail,
                                Encoding fallbackCharset)
    {
      return new Mail(MimeMessage.LoadMessage(mail,
                                              fallbackCharset));
    }

    public static Mail LoadMail(string mail,
                                EncodingSelectionCallback selectFallbackCharset)
    {
      return new Mail(MimeMessage.LoadMessage(mail,
                                              selectFallbackCharset));
    }

    public static Mail LoadMail(string mail,
                                EncodingSelectionCallback selectFallbackCharset,
                                MimeEncodedWordConverter decodeMalformedHeaderValue)
    {
      return new Mail(MimeMessage.LoadMessage(mail,
                                              selectFallbackCharset,
                                              decodeMalformedHeaderValue));
    }

    /*
     * instance members
     */
    public MimeHeaderCollection Headers {
      get { return BaseMessage.Headers; }
    }

    public MimeType MimeType {
      get { return BaseMessage.MimeType; }
    }

    public MimeMessageDisposition Disposition {
      get { return BaseMessage.Disposition; }
    }

    public string Subject {
      get { return GetHeaderValueOf("Subject"); }
    }

    public string From {
      get { return GetHeaderValueOf("From"); }
    }

    public string To {
      get { return GetHeaderValueOf("To"); }
    }

    public string DateString {
      get { return GetHeaderValueOf("Date"); }
    }

    public DateTimeOffset? Date {
      get {
        try {
          return DateTimeConvert.FromRFC822DateTimeOffsetStringNullable(GetHeaderValueOf("Date", false, true));
        }
        catch (FormatException) {
          return null;
        }
      }
    }

    public string MessageId {
      get { return GetHeaderValueOf("Message-ID"); }
    }

    public string InReplyTo {
      get { return GetHeaderValueOf("In-Reply-To"); }
    }

    public string References {
      get { return GetHeaderValueOf("References"); }
    }

    /// <returns>header value of X-Mailer or User-Agent</returns>
    public string UserAgent {
      get { return GetHeaderValueOf("X-Mailer") ?? GetHeaderValueOf("User-Agent"); }
    }

    public string ReplyTo {
      get { return GetHeaderValueOf("Reply-To"); }
    }

    public string Sender {
      get { return GetHeaderValueOf("Sender"); }
    }

#if false
    public string[] CC {
      get { return GetHeaderValueOf("CC").Split(','); }
    }

    public string[] Bcc {
      get { return GetHeaderValueOf("BCC").Split(','); }
    }
#endif

    public ReadOnlyCollection<MimeMessage> SubParts {
      get { return BaseMessage.SubParts; }
    }

    MimeMessage IMimeMessage.ParentPart {
      get { return BaseMessage.ParentPart; }
    }

    MimeMessage IMimeMessage.RootPart {
      get { return BaseMessage.RootPart; }
    }

    public MimeMessage EncapsulatedMessage {
      get { return BaseMessage.EncapsulatedMessage; }
    }

    public IEnumerable<MimeMessage> AttachmentParts {
      get {
        return BaseMessage.SubParts
          .Where(p => p.Disposition == MimeMessageDisposition.Attachment);
      }
    }

    public IEnumerable<MimeMessage> InlineMessageParts {
      get {
        return BaseMessage.SubParts
          .Where(p => p.Disposition == MimeMessageDisposition.Inline);
      }
    }

    private ReadOnlyCollection<AttachedFile> attachedFiles;

    public ReadOnlyCollection<AttachedFile> AttachedFiles {
      get {
        return LazyInitializer.EnsureInitialized(ref attachedFiles, delegate {
          return BaseMessage
            .GetFlattenSubParts()
            .Where(IsAttachedFile)
            .Select(p => new AttachedFile(this, p))
            .ToList()
            .AsReadOnly();
        });
      }
    }

    public Mail(MimeMessage baseMessage)
      : base(baseMessage)
    {
    }

    private static bool IsAttachedFile(MimeMessage part)
    {
      if (part.Disposition == MimeMessageDisposition.Attachment)
        return true;
      else if (!(part.Disposition == MimeMessageDisposition.None || part.Disposition == MimeMessageDisposition.Unknown))
        return false; // inline, etc

      if (part.MimeType.TypeEqualsIgnoreCase("multipart"))
        // multipart
        return false;
      if (part.ParentPart == null)
        // root part
        return false;

      if (MimeType.MessagePartial.EqualsIgnoreCase(part.MimeType))
        // partial message
        return false;

      if (MimeType.MultipartAlternative.EqualsIgnoreCase(part.ParentPart.MimeType))
        // alternative content part
        return false;

      if (part.ParentPart == part.RootPart &&
          MimeType.MultipartMixed.EqualsIgnoreCase(part.ParentPart.MimeType) &&
          (part.MimeType.EqualsIgnoreCase("text/plain") || part.MimeType.EqualsIgnoreCase("text/html")))
        // text part under root part
        return false;

      // other
      return true;
    }

    public void Dispose()
    {
      DisposeCore();
    }

    public string GetHeaderValueOf(string headerName)
    {
      return GetHeaderValueOf(headerName, true, false);
    }

    public string GetHeaderValueOf(string headerName, bool withParameter)
    {
      return GetHeaderValueOf(headerName, withParameter, false);
    }

    public string GetHeaderValueOf(string headerName, bool withParameter, bool removeComment)
    {
      var header = BaseMessage.Headers.GetHeaders(headerName).FirstOrDefault();

      if (header == null) {
        return null;
      }
      else {
        var ret = withParameter ? header.Value : header.GetValueWithoutParameter();

        return removeComment ? MimeHeader.RemoveCommentFoldingWhiteSpace(ret) : ret;
      }
    }

    public FileInfo[] SaveAttachedFilesToDirectory(string directory)
    {
      if (directory == null)
        throw new ArgumentNullException("directory");
      if (directory.Length == 0)
        throw ExceptionUtils.CreateArgumentMustBeNonEmptyString("directory");

      CheckDisposed();

      if (!Directory.Exists(directory))
        Directory.CreateDirectory(directory);

      return SaveAttachedFilesAs(delegate(AttachedFile attachedFile) {
        return Path.Combine(directory, attachedFile.GetDefaultFileName());
      });
    }

    public FileInfo[] SaveAttachedFilesAs(Func<AttachedFile, string> selectFileName)
    {
      if (selectFileName == null)
        throw new ArgumentNullException("selectFileName");

      return AttachedFiles.Select(delegate(AttachedFile attachedFile) {
        var file = new FileInfo(selectFileName(attachedFile));

        attachedFile.SaveAs(file);

        return file;
      }).ToArray();
    }
  }
}
